@@ -27,7 +27,7 @@ import warnings
 from pkgutil import find_loader
 from collections import OrderedDict
 from copy import copy, deepcopy
-from typing import Union  # noqa F401
+from typing import Union, Optional  # noqa F401
 
 import numpy as np
 from osgeo import gdal, gdal_array, gdalnumeric
@@ -90,7 +90,7 @@ class GeoArray(object):
         if isinstance(path_or_array, str):
             assert ' ' not in path_or_array, "The given path contains whitespaces. This is not supported by GDAL."
 
-            if not os.path.exists(path_or_array):
+            if not os.path.exists(path_or_array) and not path_or_array.startswith('/vsi'):
                 raise FileNotFoundError(path_or_array)
 
         if isinstance(path_or_array, GeoArray) or issubclass(getattr(path_or_array, '__class__'), GeoArray):
@@ -168,6 +168,8 @@ class GeoArray(object):
             self.flush_cache()  # the cached array is not useful anymore
 
         self._arr = ndarray
+        self._dtype = ndarray.dtype
+        self._shape = ndarray.shape
 
     @property
     def bandnames(self):
@@ -253,13 +255,14 @@ class GeoArray(object):
     @property
     def dtype(self):
         """Get the numpy data type of the associated image array."""
-        if self._dtype:
-            return self._dtype
-        elif self.is_inmem:
+        if self.is_inmem:
             return self.arr.dtype
         else:
-            self.set_gdalDataset_meta()
-            return self._dtype
+            if self._dtype:
+                return self._dtype
+            else:
+                self.set_gdalDataset_meta()
+                return self._dtype
 
     @property
     def geotransform(self):
@@ -376,7 +379,14 @@ class GeoArray(object):
 
     @nodata.setter
     def nodata(self, value):
-        # type: (Union[int, None]) -> None
+        # type: (Optional[bool, int, float]) -> None
+        if isinstance(value, np.bool_):
+            value = bool(value)
+        elif isinstance(value, np.integer):
+            value = int(value)
+        elif isinstance(value, np.floating):
+            value = float(value)
+
         self._nodata = value
 
         if self._metadata and value is not None:
@@ -507,13 +517,14 @@ class GeoArray(object):
 
     @property
     def metadata(self):
+        # type: () -> GDAL_Metadata
         """Return a DataFrame containing all available metadata (read from file if available).
 
         Use 'metadata[band_index].to_dict()' to get a metadata dictionary for a specific band.
         Use 'metadata.loc[row_name].to_dict()' to get all metadata values of the same key for all bands as dictionary.
         Use 'metadata.loc[row_name, band_index] = value' to set a new value.
 
-        :return:  pandas.DataFrame
+        :return:    instance of GDAL_Metadata
         """
         if self._metadata is not None:
             return self._metadata
@@ -535,7 +546,7 @@ class GeoArray(object):
                              % (self.__class__.__name__, self.__class__.__name__))
         self._metadata = meta
 
-    meta = alias_property('metadata')
+    meta = alias_property('metadata')  # type: GDAL_Metadata
 
     def __getitem__(self, given):
         if isinstance(given, (int, float, slice, np.integer, np.floating)) and self.ndim == 3:
@@ -987,65 +998,51 @@ class GeoArray(object):
 
         driver = gdal.GetDriverByName(fmt)
         if driver is None:
-            raise Exception("'%s' is not a supported GDAL driver. Refer to www.gdal.org/formats_list.html for full "
-                            "list of GDAL driver codes." % fmt)
+            raise Exception("'%s' is not a supported GDAL driver. Refer to https://gdal.org/drivers/raster/index.html "
+                            "for full list of GDAL driver codes." % fmt)
 
         if not os.path.isdir(os.path.dirname(out_path)):
             os.makedirs(os.path.dirname(out_path))
 
         envi_metadict = self.metadata.to_ENVI_metadict()
 
-        #####################
-        # write raster data #
-        #####################
+        ###########################
+        # get source GDAL dataset #
+        ###########################
+
+        ds_src: gdal.Dataset
+        ds_out: gdal.Dataset
 
         if self.is_inmem:
-            # write in-memory data
-            ds_inmem = get_GDAL_ds_inmem(self.arr,  # expects rows,columns,bands
-                                         self.geotransform, self.projection,
-                                         self._nodata)  # avoid to compute the nodata value here, so use private attrib.
-
-            # write dataset
-            ds_out = driver.CreateCopy(out_path, ds_inmem,
-                                       options=creationOptions or [])  # noqa
-
-            # # rows, columns, bands => bands, rows, columns
-            # out_arr = self.arr if self.ndim == 2 else np.swapaxes(np.swapaxes(self.arr, 0, 2), 1, 2)
-            # gdalnumeric.SaveArray(out_arr, out_path, format=fmt, prototype=ds_inmem)  # expects bands,rows,columns
-            # ds_out = gdal.Open(out_path)
-
-            del ds_inmem
-            del ds_out
+            ds_src = get_GDAL_ds_inmem(self.arr,  # expects rows,columns,bands
+                                       self.geotransform, self.projection,
+                                       self._nodata)  # avoid to compute the nodata value here, so use private attrib.
 
         else:
-            # transform linked image dataset (not in-memory) to target format
-            src_ds = gdal.Open(self.filePath)
-            if not src_ds:
-                raise Exception('Error reading file:  ' + gdal.GetLastErrorMsg())
-
+            ds_src = gdal.Open(self.filePath)
             # metadomains = {dom: src_ds.GetMetadata(dom) for dom in src_ds.GetMetadataDomainList()}
 
-            gdal.Translate(out_path, src_ds, format=fmt, creationOptions=creationOptions)
-            del src_ds
+        if not ds_src:
+            raise Exception('Error reading file:  ' + gdal.GetLastErrorMsg())
 
-        if not os.path.exists(out_path):
-            raise Exception(gdal.GetLastErrorMsg())
-
-        ################
-        # set metadata #
-        ################
-
-        # NOTE:  The dataset has to be written BEFORE metadata are added. Otherwise, metadata are not written.
-
-        ds_out = gdal.Open(out_path, gdal.GA_Update)
+        #########################################
+        # write output dataset and set metadata #
+        #########################################
 
         try:
-            if not ds_out:
-                raise Exception('Error reading file:  ' + gdal.GetLastErrorMsg())
+            # disable to write separate metadata XML files
+            os.environ['GDAL_PAM_ENABLED'] = 'NO'
 
             # ENVI #
             ########
             if fmt == 'ENVI':
+                # NOTE:  The dataset has to be written BEFORE metadata are added. Otherwise, metadata are not written.
+
+                # write ds_src to disk and re-open it to add the metadata
+                gdal.Translate(out_path, ds_src, format=fmt, creationOptions=creationOptions)
+                del ds_src
+                ds_out = gdal.Open(out_path, gdal.GA_Update)
+
                 for bidx in range(self.bands):
                     band = ds_out.GetRasterBand(bidx + 1)
 
@@ -1054,24 +1051,22 @@ class GeoArray(object):
                         band.SetDescription(bandname)
                         assert band.GetDescription() == bandname
 
-                    # NOTE: band-specific nodata values are not supported by the ENVI header format
-                    if 'nodata' in envi_metadict:
-                        nodataVal = self.metadata.band_meta['nodata'][bidx]
-                        band.SetNoDataValue(nodataVal)
-                        assert band.GetNoDataValue() == nodataVal
-
                     del band
 
                 # avoid that band names are written to global meta
                 if 'band_names' in envi_metadict:
                     del envi_metadict['band_names']
 
+                # the expected key name is 'data_ignore_value', see below
                 if 'nodata' in envi_metadict:
                     del envi_metadict['nodata']
 
                 # set data_ignore_value in case self.metadata.band_meta contains a unique nodata value
-                if 'nodata' in self.metadata.band_meta and len(set(self.metadata.band_meta['nodata'])) == 1:
-                    envi_metadict['data_ignore_value'] = str(self.metadata.band_meta['nodata'][0])
+                if 'nodata' in self.metadata.band_meta:
+                    if len(set(self.metadata.band_meta['nodata'])) == 1:
+                        envi_metadict['data_ignore_value'] = str(self.metadata.band_meta['nodata'][0])
+                    else:
+                        warnings.warn("Band-specific nodata values are not supported by the ENVI header format.")
 
                 ds_out.SetMetadata(envi_metadict, 'ENVI')
 
@@ -1079,44 +1074,60 @@ class GeoArray(object):
                     ds_out.SetDescription(envi_metadict['description'])
 
                 ds_out.FlushCache()
-                gdal.Unlink(out_path + '.aux.xml')
+                # gdal.Unlink(out_path + '.aux.xml')
 
-            elif self.metadata.all_meta:
-                # set global domain metadata
-                if self.metadata.global_meta:
-                    ds_out.SetMetadata(dict((k, repr(v)) for k, v in self.metadata.global_meta.items()))
+            else:
+                ds_out = ds_src
 
-                if 'description' in envi_metadict:
-                    ds_out.SetDescription(envi_metadict['description'])
+                # set metadata
+                if self.metadata.all_meta:
 
-                # set band domain metadata
-                bandmeta_dict = self.metadata.to_DataFrame().astype(str).to_dict()
+                    # set global domain metadata
+                    if self.metadata.global_meta:
+                        ds_out.SetMetadata(dict((k, repr(v)) for k, v in self.metadata.global_meta.items()))
 
-                for bidx in range(self.bands):
-                    band = ds_out.GetRasterBand(bidx + 1)
-                    bandmeta = bandmeta_dict[bidx].copy()
+                    if 'description' in envi_metadict:
+                        ds_out.SetDescription(envi_metadict['description'])
 
-                    # filter global metadata out
-                    bandmeta = {k: v for k, v in bandmeta.items() if k not in self.metadata.global_meta}
-                    # meta2write = dict((k, repr(v)) for k, v in self.metadata.band_meta.items() if v is not np.nan)
+                    # set band domain metadata
+                    bandmeta_dict = self.metadata.to_DataFrame().astype(str).to_dict()
 
-                    if 'band_names' in bandmeta:
-                        band.SetDescription(self.metadata.band_meta['band_names'][bidx].strip())
-                        del bandmeta['band_names']
+                    for bidx in range(self.bands):
+                        band = ds_out.GetRasterBand(bidx + 1)
+                        bandmeta = bandmeta_dict[bidx].copy()
 
-                    if 'nodata' in bandmeta:
-                        band.SetNoDataValue(self.metadata.band_meta['nodata'][bidx])
-                        del bandmeta['nodata']
+                        # filter global metadata out
+                        bandmeta = {k: v for k, v in bandmeta.items() if k not in self.metadata.global_meta}
+                        # meta2write = dict((k, repr(v)) for k, v in self.metadata.band_meta.items() if v is not np.nan)
 
-                    if bandmeta:
-                        band.SetMetadata(bandmeta)
+                        if 'band_names' in bandmeta:
+                            band.SetDescription(self.metadata.band_meta['band_names'][bidx].strip())
+                            del bandmeta['band_names']
 
-                    band.FlushCache()
-                    del band
+                        if 'nodata' in bandmeta:
+                            band.SetNoDataValue(self.metadata.band_meta['nodata'][bidx])
+                            del bandmeta['nodata']
 
-                ds_out.FlushCache()
+                        if bandmeta:
+                            band.SetMetadata(bandmeta)
+
+                        band.FlushCache()
+                        del band
+
+                    ds_out.FlushCache()
+
+                # write ds_out to disk,
+                # -> writes the in-memory array or transforms the linked dataset into the target format
+                gdal.Translate(out_path, ds_out, format=fmt, creationOptions=creationOptions)
+                del ds_src
 
         finally:
+            if 'GDAL_PAM_ENABLED' in os.environ:
+                del os.environ['GDAL_PAM_ENABLED']
+
+            if not os.path.exists(out_path):
+                raise Exception(gdal.GetLastErrorMsg())
+
             del ds_out
 
     def dump(self, out_path):
@@ -1146,6 +1157,8 @@ class GeoArray(object):
         out_nodata = in_nodata if in_nodata is not None else -9999
         if not np.can_cast(out_nodata, image2plot.dtype):
             image2plot = image2plot.astype(np.int32)
+        if image2plot.dtype == bool:
+            image2plot = image2plot.astype(int)
 
         # rotated images always have to be resampled for plotting
         if not ignore_rotation and (gt[2] or gt[4]):
@@ -1297,7 +1310,7 @@ class GeoArray(object):
                 ax = plt.gca()
 
             rows, cols = image2plot.shape[:2]
-            im = ax.imshow(image2plot.astype(int),
+            im = ax.imshow(image2plot,
                            palette,
                            interpolation=interpolation,
                            extent=(0, cols, rows, 0),
@@ -1446,6 +1459,108 @@ class GeoArray(object):
             print('2 % percentile:', np.nanpercentile(data, 2))
             print('98 % percentile:', np.nanpercentile(data, 98))
 
+    def _show_profile(self, x, y, xlabel, ylabel, title, xlim, ylim, figsize, show_nodata, return_fig):
+        from matplotlib import pyplot as plt
+
+        nd = None
+
+        if self._nodata is not None and self._nodata in y:
+            if show_nodata:
+                nd = np.ma.masked_not_equal(y, self._nodata)
+                title += ' (no-data is indicated in red)'
+            else:
+                title += ' (no-data is not shown)'
+
+            y = np.ma.masked_equal(y, self._nodata)
+
+        fig = plt.figure(figsize=figsize)
+        plt.plot(x, y, 'k')
+        if show_nodata and nd is not None:
+            plt.plot(x, nd, 'r')
+
+        plt.xlabel(xlabel)
+        plt.ylabel(ylabel)
+        plt.xlim(*xlim or (min(x), max(x)))
+        if ylim:
+            plt.ylim(*ylim)
+        plt.grid()
+        plt.title(title)
+
+        if return_fig:
+            return fig
+        else:
+            plt.show()
+
+    def show_xprofile(self, row, band, xlim=None, ylim=None, title=None, figsize=(10, 5), show_nodata=True,
+                      return_fig=False):
+        """Show an x-profile at the given row/band image position.
+
+        :param row:             image row number (counts from 0)
+        :param band:            image band number (counts from 0)
+        :param xlim:            x-axis limits to be used in the plot
+        :param ylim:            y-axis limits to be used in the plot
+        :param title:           a custom plot title
+        :param figsize:         figure size (tuple)
+        :param show_nodata:     whether to show no-data values in the plot
+        :param return_fig:      whether to return the figure instead of showing it directly
+        :return:  plt.figure
+        """
+        x = range(self.columns)
+        y = self[row, :, band]
+        title = title or f'X-Profile at row {range(self.rows)[row]}, band {range(self.bands)[band]}'
+
+        return self._show_profile(x, y, 'column', 'value', title, xlim, ylim, figsize, show_nodata, return_fig)
+
+    def show_yprofile(self, column, band, xlim=None, ylim=None, title=None, figsize=(10, 5), show_nodata=True,
+                      return_fig=False):
+        """Show a y-profile at the given column/band image position.
+
+        :param column:          image column number (counts from 0)
+        :param band:            image band number (counts from 0)
+        :param xlim:            x-axis limits to be used in the plot
+        :param ylim:            y-axis limits to be used in the plot
+        :param title:           a custom plot title
+        :param figsize:         figure size (tuple)
+        :param show_nodata:     whether to show no-data values in the plot
+        :param return_fig:      whether to return the figure instead of showing it directly
+        :return:  plt.figure
+        """
+        x = range(self.rows)
+        y = self[:, column, band]
+        title = title or f'Y-Profile at column {range(self.columns)[column]}, band {range(self.bands)[band]}'
+
+        return self._show_profile(x, y, 'row', 'value', title, xlim, ylim, figsize, show_nodata, return_fig)
+
+    def show_zprofile(self, row, column, xlim=None, ylim=None, title=None, figsize=(10, 5), show_nodata=True,
+                      return_fig=False):
+        """Show a z-profile at the given row/column image position.
+
+        :param row:             image row number (counts from 0)
+        :param column:          image column number (counts from 0)
+        :param xlim:            x-axis limits to be used in the plot
+        :param ylim:            y-axis limits to be used in the plot
+        :param title:           a custom plot title
+        :param figsize:         figure size (tuple)
+        :param show_nodata:     whether to show no-data values in the plot
+        :param return_fig:      whether to return the figure instead of showing it directly
+        :return:  plt.figure
+        """
+        if self.ndim <= 2:
+            raise RuntimeError(f'Plotting a z-profile is not possible for a {self.ndim}D array.')
+
+        if 'wavelength' in self.meta.band_meta:
+            x = self.meta.band_meta['wavelength']
+            x_label = 'wavelength'
+
+        else:
+            x = range(self.bands)
+            x_label = 'band'
+
+        y = self[row, column, :]
+        title = title or f'Z-Profile at row {range(self.rows)[row]}, column {range(self.columns)[column]}'
+
+        return self._show_profile(x, y, x_label, 'value', title, xlim, ylim, figsize, show_nodata, return_fig)
+
     def clip_to_footprint(self):
         """Clip the GeoArray instance to the outer bounds of the actual footprint."""
         self.clip_to_poly(self.footprint_poly)
@@ -1626,10 +1741,10 @@ class GeoArray(object):
 
         # apply zslice to bandnames and metadata
         if zslicing:
-            bNs_out = list(np.array(list(self.bandnames))[zslice])
+            bNs_out = list(np.array(list(self._bandnames))[zslice]) if self._bandnames else None
             _meta_out = self.metadata.get_subset(bands2extract=zslice)
         else:
-            bNs_out = list(self.bandnames)
+            bNs_out = list(self._bandnames) if self._bandnames else None
             _meta_out = self.meta
 
         sub_gA.gt = sub_gt
